/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.framework;

import java.awt.geom.Path2D;



/**
 * This class contains static methods used to create various types of paths.
 * 
 * @author Pavel Tisnovsky
 */
public class CommonPathsGenerator
{
    /**
     * Offset used to compute end points of line paths.
     */
    private static final int LINE_PATH_OFFSET = 20;

    /**
     * Default Y offset of quadratic curve end points.
     */
    private static final int DEFAULT_Y_OFFSET_FOR_QUADRATIC_CURVES = 40;

    /**
     * Create new path using Path2D.Float() which contains just one line.
     * 
     * @param image
     *            test image
     * @return created path
     */
    public static Path2D createLinePathFloat(TestImage image)
    {
        return createLinePathFloat(image.getWidth(), image.getHeight());
    }

    /**
     * Create new path using Path2D.Float() which contains just one line.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @return created path
     */
    public static Path2D createLinePathFloat(int width, int height)
    {
        Path2D path = new Path2D.Float();
        // start point
        int x1 = LINE_PATH_OFFSET;
        int y1 = LINE_PATH_OFFSET;
        // end point
        int x2 = width - LINE_PATH_OFFSET;
        int y2 = height - LINE_PATH_OFFSET;
        path.moveTo(x1, y1);
        path.lineTo(x2, y2);
        return path;
    }

    /**
     * Create new path using Path2D.Double() which contains just one line.
     * 
     * @param image
     *            test image
     * @return created path
     */
    public static Path2D createLinePathDouble(TestImage image)
    {
        return createLinePathDouble(image.getWidth(), image.getHeight());
    }

    /**
     * Create new path using Path2D.Double() which contains just one line.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @return created path
     */
    public static Path2D createLinePathDouble(int width, int height)
    {
        Path2D path = new Path2D.Double();
        // start point
        int x1 = LINE_PATH_OFFSET;
        int y1 = LINE_PATH_OFFSET;
        // end point
        int x2 = width - LINE_PATH_OFFSET;
        int y2 = height - LINE_PATH_OFFSET;
        path.moveTo(x1, y1);
        path.lineTo(x2, y2);
        return path;
    }

    /**
     * Create new path using Path2D.Float() which contains just one quadratic
     * curve.
     * 
     * @param image
     *            test image
     * @return created path
     */
    public static Path2D createQuadraticPathFloat(TestImage image)
    {
        return createQuadraticPathFloat(image.getWidth(), image.getHeight());
    }

    /**
     * Create new path using Path2D.Float() which contains just one quadratic
     * curve.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @return created path
     */
    public static Path2D createQuadraticPathFloat(int width, int height)
    {
        return createQuadraticPathFloat(width, height, 0);
    }

    /**
     * Create new path using Path2D.Float() which contains just one quadratic
     * curve.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @param yoffset
     *            vertical offset
     * @return created path
     */
    public static Path2D createQuadraticPathFloat(int width, int height, int yoffset)
    {
        Path2D path = new Path2D.Float();
        // start point
        int x1 = computeQuadraticPathX1(width);
        int y1 = computeQuadraticPathY1() + yoffset;
        // control point
        int x2 = computeQuadraticPathX2(width);
        int y2 = computeQuadraticPathY2(height) + yoffset;
        // end point
        int x3 = computeQuadraticPathX3(width);
        int y3 = computeQuadraticPathY3() + yoffset;
        path.moveTo(x1, y1);
        path.quadTo(x2, y2, x3, y3);
        return path;
    }

    /**
     * Create new path using Path2D.Double() which contains just one quadratic
     * curve.
     * 
     * @param image
     *            test image
     * @return created path
     */
    public static Path2D createQuadraticPathDouble(TestImage image)
    {
        return createQuadraticPathDouble(image.getWidth(), image.getHeight());
    }

    /**
     * Create new path using Path2D.Double() which contains just one quadratic
     * curve.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @return created path
     */
    public static Path2D createQuadraticPathDouble(int width, int height)
    {
        return createQuadraticPathDouble(width, height, 0);
    }

    /**
     * Create new path using Path2D.Double() which contains just one quadratic
     * curve.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @param yoffset
     *            vertical offset
     * @return created path
     */
    public static Path2D createQuadraticPathDouble(int width, int height, int yoffset)
    {
        Path2D path = new Path2D.Double();
        // start point
        int x1 = computeQuadraticPathX1(width);
        int y1 = computeQuadraticPathY1() + yoffset;
        // control point
        int x2 = computeQuadraticPathX2(width);
        int y2 = computeQuadraticPathY2(height) + yoffset;
        // end point
        int x3 = computeQuadraticPathX3(width);
        int y3 = computeQuadraticPathY3() + yoffset;
        path.moveTo(x1, y1);
        path.quadTo(x2, y2, x3, y3);
        return path;
    }

    /**
     * Create new path using Path2D.Float() which contains just one cubic
     * curve.
     * 
     * @param image
     *            test image
     * @return created path
     */
    public static Path2D createCubicPathFloat(TestImage image)
    {
        // compute image dimensions
        int width = image.getWidth();
        int height = image.getHeight();
        // construct path containing cubic curve
        return createCubicPathFloat(width, height);
    }

    /**
     * Create new path using Path2D.Float() which contains just one cubic
     * curve.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @return created path
     */
    public static Path2D createCubicPathFloat(int width, int height)
    {
        // construct path containing cubic curve
        return createCubicPathFloat(width, height, new int[] {1,2,3,4});
    }

    /**
     * Create new path using Path2D.Float() which contains just one cubic
     * curve.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @param pointIndexes 
     *            indexes of control point to use to draw the curve (default
     *            value should be {1,2,3,4})
     * @return created path
     */
    public static Path2D createCubicPathFloat(int width, int height, int[] pointIndexes)
    {
        // construct point set which consists of all four curve control points
        CubicCurvePointSet pointSet = new CubicCurvePointSet(width, height);

        // fill in arrays containing coordinates for all cubic curve control points
        int[] x = pointSet.getXPointArray();
        int[] y = pointSet.getYPointArray();

        // construct path containing cubic curve
        return createCubicPathFloat(width, height, x, y, pointIndexes);
    }

    /**
     * Create new path using Path2D.Float() which contains just one cubic
     * curve.
     * 
     * @param image
     *            test image
     * @param height
     *            canvas height
     * @param xarray
     *            array containing x coordinates of all four control points of
     *            cubic curve
     * @param yarray
     *            array containing y coordinates of all four control points of
     *            cubic curve
     * @param pointIndexes
     *            indexes of control point to use to draw the curve (default
     *            value should be {1,2,3,4})
     * @return
     */
    public static Path2D createCubicPathFloat(TestImage image, int[] xarray, int[] yarray, int[] pointIndexes)
    {
        // compute image dimensions
        int width = image.getWidth();
        int height = image.getHeight();
        // construct path containing cubic curve
        return createCubicPathFloat(width, height, xarray, yarray, pointIndexes);
    }

    /**
     * Create new path using Path2D.Float() which contains just one cubic
     * curve.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @param xarray
     *            array containing x coordinates of all four control points of
     *            cubic curve
     * @param yarray
     *            array containing y coordinates of all four control points of
     *            cubic curve
     * @param pointIndexes
     *            indexes of control point to use to draw the curve (default
     *            value should be {1,2,3,4})
     * @return
     */
    public static Path2D createCubicPathFloat(int width, int height, int[] xarray, int[] yarray, int[] pointIndexes)
    {
        // construct path containing cubic curve
        return createCubicPathFloat(width, height, xarray, yarray, pointIndexes, 0);
    }

    /**
     * Create new path using Path2D.Float() which contains just one cubic curve.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @param xarray
     *            array containing x coordinates of all four control points of
     *            cubic curve
     * @param yarray
     *            array containing y coordinates of all four control points of
     *            cubic curve
     * @param pointIndexes
     *            indexes of control point to use to draw the curve (default
     *            value should be {1,2,3,4})
     * @param yoffset
     *            offset for all y-coordinates
     * @return
     */
    public static Path2D createCubicPathFloat(int width, int height, int[] xarray, int[] yarray, int[] pointIndexes, int yoffset)
    {
        // construct path containing cubic curve
        Path2D path = new Path2D.Float();
        path.moveTo(xarray[pointIndexes[0]-1], yarray[pointIndexes[0]-1] + yoffset);
        path.curveTo(xarray[pointIndexes[1]-1], yarray[pointIndexes[1]-1] + yoffset,
                     xarray[pointIndexes[2]-1], yarray[pointIndexes[2]-1] + yoffset,
                     xarray[pointIndexes[3]-1], yarray[pointIndexes[3]-1] + yoffset);
        return path;
    }

    /**
     * Create new path using Path2D.Float() which contains just one cubic curve.
     * 
     * @param image
     *            test image
     * @param height
     *            canvas height
     * @param xarray
     *            array containing x coordinates of all four control points of
     *            cubic curve
     * @param yarray
     *            array containing y coordinates of all four control points of
     *            cubic curve
     * @param pointIndexes
     *            indexes of control point to use to draw the curve (default
     *            value should be {1,2,3,4})
     * @param yoffset
     *            offset for all y-coordinates
     * @return
     */
    public static Path2D createCubicPathFloat(TestImage image, int[] xarray, int[] yarray, int[] pointIndexes, int yoffset)
    {
        // image dimensions
        int width = image.getWidth();
        int height = image.getHeight();
        // create and return the path
        return createCubicPathFloat(width, height, xarray, yarray, pointIndexes, yoffset);
    }

    /**
     * Create new path using Path2D.Float() which contains just one cubic
     * curve.
     * 
     * @param image
     *            test image
     * @return created path
     */
    public static Path2D createCubicPathDouble(TestImage image)
    {
        // compute image dimensions
        int width = image.getWidth();
        int height = image.getHeight();
        // construct path containing cubic curve
        return createCubicPathDouble(width, height);
    }

    /**
     * Create new path using Path2D.Double() which contains just one cubic
     * curve.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @return created path
     */
    public static Path2D createCubicPathDouble(int width, int height)
    {
        // construct path containing cubic curve
        return createCubicPathDouble(width, height, new int[] {1,2,3,4});
    }

    /**
     * Create new path using Path2D.Double() which contains just one cubic
     * curve.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @param pointIndexes 
     *            indexes of control point to use to draw the curve (default
     *            value should be {1,2,3,4})
     * @return created path
     */
    public static Path2D createCubicPathDouble(int width, int height, int[] pointIndexes)
    {
        // construct point set which consists of all four curve control points
        CubicCurvePointSet pointSet = new CubicCurvePointSet(width, height);

        // fill in arrays containing coordinates for all cubic curve control points
        int[] x = pointSet.getXPointArray();
        int[] y = pointSet.getYPointArray();

        // construct path containing cubic curve
        return createCubicPathDouble(width, height, x, y, pointIndexes);
    }

    /**
     * Create new path using Path2D.Double() which contains just one cubic
     * curve.
     * 
     * @param image
     *            test image
     * @param height
     *            canvas height
     * @param xarray
     *            array containing x coordinates of all four control points of
     *            cubic curve
     * @param yarray
     *            array containing y coordinates of all four control points of
     *            cubic curve
     * @param pointIndexes
     *            indexes of control point to use to draw the curve (default
     *            value should be {1,2,3,4})
     * @return
     */
    public static Path2D createCubicPathDouble(TestImage image, int[] xarray, int[] yarray, int[] pointIndexes)
    {
        // compute image dimensions
        int width = image.getWidth();
        int height = image.getHeight();
        // construct path containing cubic curve
        return createCubicPathDouble(width, height, xarray, yarray, pointIndexes);
    }

    /**
     * Create new path using Path2D.Double() which contains just one cubic
     * curve.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @param xarray
     *            array containing x coordinates of all four control points of
     *            cubic curve
     * @param yarray
     *            array containing y coordinates of all four control points of
     *            cubic curve
     * @param pointIndexes
     *            indexes of control point to use to draw the curve (default
     *            value should be {1,2,3,4})
     * @return
     */
    public static Path2D createCubicPathDouble(int width, int height, int[] xarray, int[] yarray, int[] pointIndexes)
    {
        return createCubicPathDouble(width, height, xarray, yarray, pointIndexes, 0);
    }

    /**
     * Create new path using Path2D.Double() which contains just one cubic
     * curve.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @param xarray
     *            array containing x coordinates of all four control points of
     *            cubic curve
     * @param yarray
     *            array containing y coordinates of all four control points of
     *            cubic curve
     * @param pointIndexes
     *            indexes of control point to use to draw the curve (default
     *            value should be {1,2,3,4})
     * @param yoffset
     *            offset for all y-coordinates
     * @return
     */
    public static Path2D createCubicPathDouble(int width, int height, int[] xarray, int[] yarray, int[] pointIndexes, int yoffset)
    {
        // construct path containing cubic curve
        Path2D path = new Path2D.Double();
        path.moveTo(xarray[pointIndexes[0]-1], yarray[pointIndexes[0]-1] + yoffset);
        path.curveTo(xarray[pointIndexes[1]-1], yarray[pointIndexes[1]-1] + yoffset,
                     xarray[pointIndexes[2]-1], yarray[pointIndexes[2]-1] + yoffset,
                     xarray[pointIndexes[3]-1], yarray[pointIndexes[3]-1] + yoffset);
        return path;
    }

    /**
     * Create new path using Path2D.Double() which contains just one cubic curve.
     * 
     * @param image
     *            test image
     * @param height
     *            canvas height
     * @param xarray
     *            array containing x coordinates of all four control points of
     *            cubic curve
     * @param yarray
     *            array containing y coordinates of all four control points of
     *            cubic curve
     * @param pointIndexes
     *            indexes of control point to use to draw the curve (default
     *            value should be {1,2,3,4})
     * @param yoffset
     *            offset for all y-coordinates
     * @return
     */
    public static Path2D createCubicPathDouble(TestImage image, int[] xarray, int[] yarray, int[] pointIndexes, int yoffset)
    {
        // image dimensions
        int width = image.getWidth();
        int height = image.getHeight();
        // create and return the path
        return createCubicPathDouble(width, height, xarray, yarray, pointIndexes, yoffset);
    }

    /**
     * Create simple new path using Path2D.Float() which contains just lines.
     * 
     * @param image
     *            test image
     * @return created path
     */
    public static Path2D createClosedPathFloat(TestImage image)
    {
        return createClosedPathFloat(image.getWidth(), image.getHeight());
    }

    /**
     * Create simple new path using Path2D.Float() which contains just lines.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @return created path
     */
    public static Path2D createClosedPathFloat(int width, int height)
    {
        Path2D path = new Path2D.Float();
        // 1st vertex
        int x1 = LINE_PATH_OFFSET;
        int y1 = LINE_PATH_OFFSET;
        // 2nd vertex
        int x2 = width - LINE_PATH_OFFSET;
        int y2 = y1;
        // 3rd vertex
        int x3 = x2;
        int y3 = height - LINE_PATH_OFFSET;
        // 4rd vertex
        int x4 = x1;
        int y4 = y3;
        path.moveTo(x1, y1);
        path.lineTo(x2, y2);
        path.lineTo(x3, y3);
        path.lineTo(x4, y4);
        path.closePath();
        return path;
    }

    /**
     * Create simple new path using Path2D.Double() which contains just lines.
     * 
     * @param image
     *            test image
     * @return created path
     */
    public static Path2D createClosedPathDouble(TestImage image)
    {
        return createClosedPathFloat(image.getWidth(), image.getHeight());
    }

    /**
     * Create simple new path using Path2D.Double() which contains just lines.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @return created path
     */
    public static Path2D createClosedPathDouble(int width, int height)
    {
        Path2D path = new Path2D.Double();
        // 1st vertex
        int x1 = LINE_PATH_OFFSET;
        int y1 = LINE_PATH_OFFSET;
        // 2nd vertex
        int x2 = width - LINE_PATH_OFFSET;
        int y2 = y1;
        // 3rd vertex
        int x3 = x2;
        int y3 = height - LINE_PATH_OFFSET;
        // 4rd vertex
        int x4 = x1;
        int y4 = y3;
        path.moveTo(x1, y1);
        path.lineTo(x2, y2);
        path.lineTo(x3, y3);
        path.lineTo(x4, y4);
        path.closePath();
        return path;
    }

    /**
     * 
     * @param image
     * @return
     */
    public static Path2D createClosedPathContainingQuadraticSegmentFloat(TestImage image)
    {
        return createClosedPathContainingCubicSegmentFloat(image.getWidth(), image.getHeight());
    }

    /**
     * 
     * @param width
     * @param height
     * @return
     */
    public static Path2D createClosedPathContainingQuadraticSegmentFloat(int width, int height)
    {
        Path2D path = new Path2D.Float();
        // 1st vertex
        int x1 = LINE_PATH_OFFSET;
        int y1 = LINE_PATH_OFFSET;
        // 2nd vertex
        int x2 = width - LINE_PATH_OFFSET;
        int y2 = y1;
        // 3rd vertex
        int x3 = x2;
        int y3 = height - LINE_PATH_OFFSET;
        // 4rd vertex
        int x4 = x1;
        int y4 = y3;
        path.moveTo(x1, y1);
        path.lineTo(x2, y2);
        path.quadTo(x3, y3, x4, y4);
        path.closePath();
        return path;
    }

    /**
     * 
     * @param image
     * @return
     */
    public static Path2D createClosedPathContainingCubicSegmentFloat(TestImage image)
    {
        return createClosedPathContainingCubicSegmentFloat(image.getWidth(), image.getHeight());
    }

    /**
     * 
     * @param width
     * @param height
     * @return
     */
    public static Path2D createClosedPathContainingCubicSegmentFloat(int width, int height)
    {
        Path2D path = new Path2D.Float();
        // 1st vertex
        int x1 = LINE_PATH_OFFSET;
        int y1 = LINE_PATH_OFFSET;
        // 2nd vertex
        int x2 = width - LINE_PATH_OFFSET;
        int y2 = y1;
        // 3rd vertex
        int x3 = x2;
        int y3 = height - LINE_PATH_OFFSET;
        // 4rd vertex
        int x4 = x1;
        int y4 = y3;
        path.moveTo(x1, y1);
        path.curveTo(x2, y2, x3, y3, x4, y4);
        path.closePath();
        return path;
    }

    /**
     * 
     * @param image
     * @return
     */
    public static Path2D createClosedPathContainingQuadraticSegmentDouble(TestImage image)
    {
        return createClosedPathContainingQuadraticSegmentDouble(image.getWidth(), image.getHeight());
    }

    /**
     * 
     * @param width
     * @param height
     * @return
     */
    public static Path2D createClosedPathContainingQuadraticSegmentDouble(int width, int height)
    {
        Path2D path = new Path2D.Double();
        // 1st vertex
        int x1 = LINE_PATH_OFFSET;
        int y1 = LINE_PATH_OFFSET;
        // 2nd vertex
        int x2 = width - LINE_PATH_OFFSET;
        int y2 = y1;
        // 3rd vertex
        int x3 = x2;
        int y3 = height - LINE_PATH_OFFSET;
        // 4rd vertex
        int x4 = x1;
        int y4 = y3;
        path.moveTo(x1, y1);
        path.lineTo(x2, y2);
        path.quadTo(x3, y3, x4, y4);
        path.closePath();
        return path;
    }

    /**
     * 
     * @param image
     * @return
     */
    public static Path2D createClosedPathContainingCubicSegmentDouble(TestImage image)
    {
        return createClosedPathContainingCubicSegmentDouble(image.getWidth(), image.getHeight());
    }

    /**
     * 
     * @param width
     * @param height
     * @return
     */
    public static Path2D createClosedPathContainingCubicSegmentDouble(int width, int height)
    {
        Path2D path = new Path2D.Float();
        // 1st vertex
        int x1 = LINE_PATH_OFFSET;
        int y1 = LINE_PATH_OFFSET;
        // 2nd vertex
        int x2 = width - LINE_PATH_OFFSET;
        int y2 = y1;
        // 3rd vertex
        int x3 = x2;
        int y3 = height - LINE_PATH_OFFSET;
        // 4rd vertex
        int x4 = x1;
        int y4 = y3;
        path.moveTo(x1, y1);
        path.curveTo(x2, y2, x3, y3, x4, y4);
        path.closePath();
        return path;
    }

    /**
     * Create new path using Path2D.Float() which contains just lines.
     * 
     * @param image
     *            test image
     * @return created path
     */
    public static Path2D createCrossedClosedPathFloat(TestImage image)
    {
        return createCrossedClosedPathFloat(image.getWidth(), image.getHeight());
    }

    /**
     * Create new path using Path2D.Float() which contains just lines.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @return created path
     */
    public static Path2D createCrossedClosedPathFloat(int width, int height)
    {
        Path2D path = new Path2D.Float();
        // 1st vertex
        int x1 = LINE_PATH_OFFSET;
        int y1 = LINE_PATH_OFFSET;
        // 2nd vertex
        int x2 = width - LINE_PATH_OFFSET;
        int y2 = height - LINE_PATH_OFFSET;
        // 3rd vertex
        int x3 = x2;
        int y3 = y1;
        // 4rd vertex
        int x4 = x1;
        int y4 = y2;
        path.moveTo(x1, y1);
        path.lineTo(x2, y2);
        path.lineTo(x3, y3);
        path.lineTo(x4, y4);
        path.closePath();
        return path;
    }

    /**
     * Create new path using Path2D.Double() which contains just lines.
     * 
     * @param image
     *            test image
     * @return created path
     */
    public static Path2D createCrossedClosedPathDouble(TestImage image)
    {
        return createCrossedClosedPathDouble(image.getWidth(), image.getHeight());
    }

    /**
     * Create new path using Path2D.Double() which contains just lines.
     * 
     * @param width
     *            canvas width
     * @param height
     *            canvas height
     * @return created path
     */
    public static Path2D createCrossedClosedPathDouble(int width, int height)
    {
        Path2D path = new Path2D.Double();
        // 1st vertex
        int x1 = LINE_PATH_OFFSET;
        int y1 = LINE_PATH_OFFSET;
        // 2nd vertex
        int x2 = width - LINE_PATH_OFFSET;
        int y2 = height - LINE_PATH_OFFSET;
        // 3rd vertex
        int x3 = x2;
        int y3 = y1;
        // 4rd vertex
        int x4 = x1;
        int y4 = y2;
        path.moveTo(x1, y1);
        path.lineTo(x2, y2);
        path.lineTo(x3, y3);
        path.lineTo(x4, y4);
        path.closePath();
        return path;
    }

    public static Path2D createCrossedClosedPathContainingQuadraticSegmentFloat(TestImage image)
    {
        return createCrossedClosedPathContainingQuadraticSegmentFloat(image.getWidth(), image.getWidth());
    }

    public static Path2D createCrossedClosedPathContainingQuadraticSegmentFloat(int width, int height)
    {
        Path2D path = new Path2D.Float();
        // 1st vertex
        int x1 = LINE_PATH_OFFSET;
        int y1 = LINE_PATH_OFFSET;
        // 2nd vertex
        int x2 = width - LINE_PATH_OFFSET;
        int y2 = height - LINE_PATH_OFFSET * 4;
        // 3rd vertex
        int x3 = x2;
        int y3 = y1;
        // 4rd vertex
        int x4 = x1;
        int y4 = y2 - 100;
        path.moveTo(x2, y1);
        path.quadTo(x4, y2, x1, y3);
        path.lineTo(x3, y4);
        path.closePath();
        return path;
    }

    public static Path2D createCrossedClosedPathContainingQuadraticSegmentDouble(TestImage image)
    {
        return createCrossedClosedPathContainingQuadraticSegmentDouble(image.getWidth(), image.getWidth());
    }

    public static Path2D createCrossedClosedPathContainingQuadraticSegmentDouble(int width, int height)
    {
        Path2D path = new Path2D.Double();
        // 1st vertex
        int x1 = LINE_PATH_OFFSET;
        int y1 = LINE_PATH_OFFSET;
        // 2nd vertex
        int x2 = width - LINE_PATH_OFFSET;
        int y2 = height - LINE_PATH_OFFSET * 4;
        // 3rd vertex
        int x3 = x2;
        int y3 = y1;
        // 4rd vertex
        int x4 = x1;
        int y4 = y2 - 100;
        path.moveTo(x2, y1);
        path.quadTo(x3, y2, x4, y3);
        path.lineTo(x2, y4);
        path.closePath();
        return path;
    }

    public static Path2D createCrossedClosedPathContainingCubicSegmentFloat(TestImage image)
    {
        return createCrossedClosedPathContainingCubicSegmentFloat(image.getWidth(), image.getWidth());
    }

    public static Path2D createCrossedClosedPathContainingCubicSegmentFloat(int width, int height)
    {
        Path2D path = new Path2D.Double();
        // 1st vertex
        int x1 = LINE_PATH_OFFSET;
        int y1 = LINE_PATH_OFFSET;
        // 2nd vertex
        int x2 = width - LINE_PATH_OFFSET;
        int y2 = height - LINE_PATH_OFFSET * 4;
        // 3rd vertex
        int x3 = x2;
        int y3 = y1;
        // 4rd vertex
        int x4 = x1;
        int y4 = y2;
        path.moveTo(x1, y1);
        path.curveTo(x2 + 100, y2, x4 - 100, y4, x3, y3);
        path.lineTo((x1 + x2) / 2, y2 - 100);
        path.closePath();
        return path;
    }

    public static Path2D createCrossedClosedPathContainingCubicSegmentDouble(TestImage image)
    {
        return createCrossedClosedPathContainingCubicSegmentDouble(image.getWidth(), image.getWidth());
    }

    public static Path2D createCrossedClosedPathContainingCubicSegmentDouble(int width, int height)
    {
        return null;
    }

    /**
     * Compute X coordinate of first curve end point.
     * 
     * @param width
     *            canvas width
     * @return X coordinate of first curve end point.
     */
    public static int computeQuadraticPathX1(int width)
    {
        return width >> 2;
    }

    /**
     * Compute X coordinate of curve control point.
     *
     * @param width
     *            canvas width
     * @return X coordinate of curve control point.
     */
    public static int computeQuadraticPathX2(int width)
    {
        return width >> 1;
    }

    /**
     * Compute X coordinate of second curve end point.
     * 
     * @param width
     *            canvas width
     * @return X coordinate of second curve end point.
     */
    public static int computeQuadraticPathX3(int width)
    {
        return width * 3 / 4;
    }

    /**
     * Compute Y coordinate of first curve end point.
     *
     * @return Y coordinate of first curve end point.
     */
    public static int computeQuadraticPathY1()
    {
        return DEFAULT_Y_OFFSET_FOR_QUADRATIC_CURVES;
    }

    /**
     * Compute Y coordinate of curve control point.
     *
     * @param height
     *            canvas height
     * @return Y coordinate of curve control point.
     */
    public static int computeQuadraticPathY2(int height)
    {
        return DEFAULT_Y_OFFSET_FOR_QUADRATIC_CURVES + height * 2 / 3;
    }

    /**
     * Compute Y coordinate of second curve end point.
     *
     * @return Y coordinate of second curve end point.
     */
    public static int computeQuadraticPathY3()
    {
        return DEFAULT_Y_OFFSET_FOR_QUADRATIC_CURVES;
    }

}
