/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;



import org.gfxtest.framework.GfxTest;
import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.GraphicsPrimitive;
import org.gfxtest.framework.annotations.GraphicsPrimitives;
import org.gfxtest.framework.annotations.RenderStyle;
import org.gfxtest.framework.annotations.RenderStyles;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;



/**
 * This test check the rendering of antialiased circles and ellipses. All sample
 * images and test images are zoomed to show the pixel-wide differences between
 * both renderers. Zoom factor is set to 16.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.RENDER_TEST)
@GraphicsPrimitive(GraphicsPrimitives.ELLIPSE)
@RenderStyle(RenderStyles.NORMAL_AA)
@Transformation(Transformations.NONE)
@Zoom(16)
public class AAEllipses extends GfxTest
{

    /**
     * Draw circle onto the image using given color and also highlights circle
     * center point and four points at quadrants. Odd width and height is used
     * for the test circle.
     * 
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param xc
     *            x-coordination of circle center
     * @param yc
     *            y-coordination of circle center
     * @param radius
     *            radius of circle
     * @param color
     *            circle color
     */
    private static void drawCircleOddSize(BufferedImage image, Graphics2D graphics2d, int xc, int yc, int radius, Color color)
    {
        // rectangle around the circle
        int x1 = xc - radius;
        int y1 = yc - radius;
        int x2 = xc + radius;
        int y2 = yc + radius;

        // circle size
        int size = 1 + radius * 2;

        // draw circle
        graphics2d.setColor(color);
        graphics2d.drawOval(x1, y1, size, size);

        // draw interesting points for the circle
        renderCircleCenter(image, xc, yc);
        renderCircleQuadrant(image, xc, yc, x1, y1, x2, y2);
    }

    /**
     * Draw circle onto the image using given color and also highlights circle
     * center point and four points at quadrants. Even width and height is used
     * for the test circle.
     * 
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param xc
     *            x-coordination of circle center
     * @param yc
     *            y-coordination of circle center
     * @param radius
     *            radius of circle
     * @param color
     *            circle color
     */
    private static void drawCircleEvenSize(BufferedImage image, Graphics2D graphics2d, int xc, int yc, int radius, Color color)
    {
        // rectangle around the circle
        int x1 = xc - radius;
        int y1 = yc - radius;
        int x2 = xc + radius;
        int y2 = yc + radius;

        // circle size
        int size = radius * 2;

        // draw circle
        graphics2d.setColor(color);
        graphics2d.drawOval(x1, y1, size, size);

        // draw interesting points for the circle
        renderCircleCenter(image, xc, yc);
        renderCircleQuadrant(image, xc, yc, x1, y1, x2, y2);
    }

    /**
     * Draw pixel which is in the center of circle.
     *
     * @param image
     *            image to which line is to be drawn
     * @param xc
     *            x-coordination of circle center
     * @param yc
     *            y-coordination of circle center
     */
    private static void renderCircleCenter(BufferedImage image, int xc, int yc)
    {
        // pixes at the center of circle
        image.setRGB(xc, yc, 0xff0000);
    }

    /**
     * Draw pixel which is in the circle quadrant.
     * 
     * @param image
     *            image to which line is to be drawn
     * @param xc
     *            x-coordination of circle center
     * @param yc
     *            y-coordination of circle center
     * @param x1
     *            leftmost part of circle
     * @param y1
     *            topmost part of circle
     * @param x2
     *            rightmost part of circle
     * @param y2
     *            undermost part of circle
     */
    private static void renderCircleQuadrant(BufferedImage image, int xc, int yc, int x1, int y1, int x2, int y2)
    {
        renderCircleCenter(image, x1, yc);
        renderCircleCenter(image, x2, yc);
        renderCircleCenter(image, xc, y1);
        renderCircleCenter(image, xc, y2);
    }

    /**
     * Draw circles with odd size (width and height).
     *
     * @param image
     *            test image
     * @param graphics2d
     *            graphics canvas
     */
    private void drawCirclesOddSize(TestImage image, Graphics2D graphics2d)
    {
        // compute center of circles
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        // image used for rendering
        final BufferedImage bufferedImage = image.getImage();
        // draw circles
        drawCircleOddSize(bufferedImage, graphics2d, xc, yc, 2, Color.BLACK);
        drawCircleOddSize(bufferedImage, graphics2d, xc, yc, 4, Color.YELLOW);
        drawCircleOddSize(bufferedImage, graphics2d, xc, yc, 6, Color.MAGENTA);
        drawCircleOddSize(bufferedImage, graphics2d, xc, yc, 9, Color.BLUE);
        drawCircleOddSize(bufferedImage, graphics2d, xc, yc, 12, Color.GRAY);
    }

    /**
     * Draw circles with even size (width and height).
     * @param image
     *            test image
     * @param graphics2d
     *            graphics canvas
     */
    private void drawCirclesEvenSize(TestImage image, Graphics2D graphics2d)
    {
        // compute center of circles
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        // image used for rendering
        final BufferedImage bufferedImage = image.getImage();
        // draw circles
        drawCircleEvenSize(bufferedImage, graphics2d, xc, yc, 2, Color.BLACK);
        drawCircleEvenSize(bufferedImage, graphics2d, xc, yc, 4, Color.YELLOW);
        drawCircleEvenSize(bufferedImage, graphics2d, xc, yc, 6, Color.MAGENTA);
        drawCircleEvenSize(bufferedImage, graphics2d, xc, yc, 9, Color.BLUE);
        drawCircleEvenSize(bufferedImage, graphics2d, xc, yc, 12, Color.GRAY);
    }

    /**
     * This method just enables antialiasing for the following rendering
     * operation.
     * 
     * @param graphics2d
     *            graphics canvas
     */
    private void setAA1(Graphics2D graphics)
    {
        graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    }

    /**
     * This method just enables antialiasing and fractional metrics for the
     * following rendering operation.
     * 
     * @param graphics2d
     *            graphics canvas
     */
    private void setAA2(Graphics2D graphics)
    {
        graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        graphics.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
    }

    /**
     * Test rendering of circles with various sizes, but the sizes are all odd.
     * 
     * @param image
     *            test image
     * @param graphics2d
     *            graphics canvas
     * @return test result
     */
    public TestResult testCircleOddSizeNoAA(TestImage image, Graphics2D graphics2d)
    {
        // draw test circles
        drawCirclesOddSize(image, graphics2d);
        return TestResult.PASSED;
    }

    /**
     * Test rendering of circles with various sizes, but the sizes are all even.
     * 
     * @param image
     *            test image
     * @param graphics2d
     *            graphics canvas
     * @return test result
     */
    public TestResult testCircleEvenSizeNoAA(TestImage image, Graphics2D graphics2d)
    {
        // draw test circles
        drawCirclesEvenSize(image, graphics2d);
        return TestResult.PASSED;
    }

    /**
     * Test rendering of circles with various sizes, but the sizes are all odd.
     * 
     * @param image
     *            test image
     * @param graphics2d
     *            graphics canvas
     * @return test result
     */
    public TestResult testCircleOddSizeAA1(TestImage image, Graphics2D graphics2d)
    {
        // enable antialiasing
        setAA1(graphics2d);
        // draw test circles
        drawCirclesOddSize(image, graphics2d);
        return TestResult.PASSED;
    }

    /**
     * Test rendering of circles with various sizes, but the sizes are all even.
     * 
     * @param image
     *            test image
     * @param graphics2d
     *            graphics canvas
     * @return test result
     */
    public TestResult testCircleEvenSizeAA1(TestImage image, Graphics2D graphics2d)
    {
        // enable antialiasing
        setAA1(graphics2d);
        // draw test circles
        drawCirclesEvenSize(image, graphics2d);
        return TestResult.PASSED;
    }

    /**
     * Test rendering of circles with various sizes, but the sizes are all odd.
     * 
     * @param image
     *            test image
     * @param graphics2d
     *            graphics canvas
     * @return test result
     */
    public TestResult testCircleOddSizeAA2(TestImage image, Graphics2D graphics2d)
    {
        // enable antialiasing and fractional metrics
        setAA2(graphics2d);
        // draw test circles
        drawCirclesOddSize(image, graphics2d);
        return TestResult.PASSED;
    }

    /**
     * Test rendering of circles with various sizes, but the sizes are all even.
     * 
     * @param image
     *            test image
     * @param graphics2d
     *            graphics canvas
     * @return test result
     */
    public TestResult testCircleEvenSizeAA2(TestImage image, Graphics2D graphics2d)
    {
        // enable antialiasing and fractional metrics
        setAA2(graphics2d);
        // draw test circles
        drawCirclesEvenSize(image, graphics2d);
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     *
     * @param args not used in this case
     */
    public static void main(String[] args)
    {
        new AAEllipses().runTestSuite(args);
    }

}
