/*
  Java gfx-test framework

   Copyright (C) 2010, 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
 */

package org.gfxtest.testsuites;

import java.awt.BasicStroke;
import java.awt.Graphics2D;



import org.gfxtest.framework.CommonClippingOperations;
import org.gfxtest.framework.CommonPathsGenerator;
import org.gfxtest.framework.CommonRenderingStyles;
import org.gfxtest.framework.GfxTest;
import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.GraphicsPrimitive;
import org.gfxtest.framework.annotations.GraphicsPrimitives;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;



/**
 * This test check if clipping is working correctly for a path which is clipped
 * by a rectangle area. Various path and various rendering styles are used for
 * rendering.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.RENDER_TEST)
@Transformation(Transformations.NONE)
@GraphicsPrimitive(GraphicsPrimitives.PATH)
@Zoom(1)
public class ClippingPathByRectangleArea extends GfxTest
{
    private static final float[] DOTTED_PATTERN = new float[] {1,40};

    /**
     * Prepare canvas for the rendering.
     *
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void basicSetupForRendering(TestImage image, Graphics2D graphics2d)
    {
        // create clip area
        CommonClippingOperations.createClipUsingRectangleArea(image, graphics2d);
    }

    /**
     * Draw path consisting of one line only clipped by rectangle area.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawLinePathClippedByRectangleArea(TestImage image, Graphics2D graphics2d)
    {
        // prepare canvas for the rendering
        basicSetupForRendering(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the line path
        graphics2d.draw(CommonPathsGenerator.createLinePathFloat(image));
    }

    /**
     * Draw quadratic path clipped by rectangle area.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawQuadraticPathClippedByRectangleArea(TestImage image, Graphics2D graphics2d)
    {
        // prepare canvas for the rendering
        basicSetupForRendering(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the quadratic path
        graphics2d.draw(CommonPathsGenerator.createQuadraticPathFloat(image));
    }

    /**
     * Draw cubic path clipped by rectangle area.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawCubicPathClippedByRectangleArea(TestImage image, Graphics2D graphics2d)
    {
        // prepare canvas for the rendering
        basicSetupForRendering(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the cubic path
        graphics2d.draw(CommonPathsGenerator.createCubicPathFloat(image));
    }

    /**
     * Draw closed path clipped by rectangle area.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawClosedPathClippedByRectangleArea(TestImage image, Graphics2D graphics2d)
    {
        // prepare canvas for the rendering
        basicSetupForRendering(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the closed path
        graphics2d.draw(CommonPathsGenerator.createClosedPathFloat(image));
    }

    /**
     * Draw crossed closed path clipped by rectangle area.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawCrossedClosedPathClippedByRectangleArea(TestImage image, Graphics2D graphics2d)
    {
        // prepare canvas for the rendering
        basicSetupForRendering(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the crossed closed path
        graphics2d.draw(CommonPathsGenerator.createCrossedClosedPathFloat(image));
    }

    /**
     * Check if line path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRectangleAreaStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRectangleAreaStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRectangleAreaStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRectangleAreaStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRectangleAreaStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRectangleAreaZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRectangleAreaZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRectangleAreaZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRectangleAreaZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRectangleAreaZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRectangleAreaThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRectangleAreaThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRectangleAreaThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRectangleAreaThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRectangleAreaThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with extra thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRectangleAreaExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with extra thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRectangleAreaExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with extra thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRectangleAreaExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with extra thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRectangleAreaExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with extra thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRectangleAreaExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRectangleAreaDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRectangleAreaDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRectangleAreaDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRectangleAreaDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed crossed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRectangleAreaDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRectangleAreaDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRectangleAreaDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRectangleAreaDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRectangleAreaDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRectangleAreaDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRectangleAreaDashedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRectangleAreaDashedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRectangleAreaDashedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRectangleAreaDashedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRectangleAreaDashedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRectangleAreaDashedZeroThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRectangleAreaDashedZeroThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRectangleAreaDashedZeroThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRectangleAreaDashedZeroThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a rectangle area. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRectangleAreaDashedZeroThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with normal stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRectangleAreaDottedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        //CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawLinePathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with normal stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRectangleAreaDottedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        //CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawQuadraticPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with normal stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRectangleAreaDottedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        //CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawCubicPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with normal stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRectangleAreaDottedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        //CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with normal stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRectangleAreaDottedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        //CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRectangleAreaDottedZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawLinePathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRectangleAreaDottedZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawQuadraticPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRectangleAreaDottedZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawCubicPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRectangleAreaDottedZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRectangleAreaDottedZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRectangleAreaDottedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawLinePathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRectangleAreaDottedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawQuadraticPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRectangleAreaDottedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawCubicPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRectangleAreaDottedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRectangleAreaDottedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with extra thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRectangleAreaDottedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawLinePathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with extra thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRectangleAreaDottedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawQuadraticPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with extra thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRectangleAreaDottedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawCubicPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with extra thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRectangleAreaDottedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a rectangle area. Path is rendered
     * using dotted stroke paint with extra thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRectangleAreaDottedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip rectangle
        CommonClippingOperations.renderClipRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRectangleArea(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     * 
     * @param args
     *            graphics test configuration
     */
    public static void main(String[] args)
    {
        new ClippingPathByRectangleArea().runTestSuite(args);
    }
}
